# -*- coding: utf-8 -*-
# Structure modeled after Jamendo dumps
# Copyright (C) 2010  Sylvain Beucler
#
# This file is part of Djamengo.
# 
# Djamengo is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# Djamengo is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# Author: Sylvain Beucler

from django.db import models
import urllib
from django.core import urlresolvers

class Artist(models.Model):
    class Meta:
        ordering = ['name',]

    name = models.CharField(max_length=255, db_index=True, blank=True, null=True, help_text="display name")
    url = models.URLField(verify_exists=False, blank=True, null=True, help_text="jamendo url to artist")
    image = models.URLField(verify_exists=False, blank=True, null=True, help_text="artist image url")
    mbgid = models.CharField(max_length=36, db_index=True, blank=True, null=True, \
                                 help_text="musicbrainz ID (may be blank)")

    # structure about the main location of the artist
    location_country = models.CharField(max_length=3, db_index=True, blank=True, null=True, \
                                            help_text="ISO code of the country")
    location_state = models.CharField(max_length=3, blank=True, null=True,
      help_text="ISO Country subdivision (states in the US, \"departments\" in France,  ...)")
    location_city = models.CharField(max_length=255, blank=True, null=True)
    location_latitude = models.FloatField(blank=True, null=True)
    location_longitude = models.FloatField(blank=True, null=True)

    def __unicode__(self):
        return self.name

class Album(models.Model):
    class Meta:
        ordering = ['name',]

    artist = models.ForeignKey("Artist")
    name = models.CharField(max_length=255, db_index=True, blank=True, null=True, help_text="display name")
    url = models.URLField(verify_exists=False, blank=True, null=True, help_text="jamendo url to album")
    id3genre = models.IntegerField(db_index=True, blank=True, null=True)
    mbgid = models.CharField(max_length=36, db_index=True, blank=True, null=True, \
                                 help_text="musicbrainz ID (may be blank)")
    license_artwork = models.URLField(verify_exists=False, blank=True, null=True)
    releasedate = models.DateTimeField(blank=True, null=True, db_index=True, help_text="ISO-formatted date")
    filename = models.CharField(max_length=255, db_index=True, blank=True, null=True,
      help_text="Usual directory name without extension")

    def url_html(self):
        return "<a href='%s'>%s</a>" % (self.url, self.url)
    url_html.allow_tags = True
    url_html.admin_order_field = 'url'

    def ogg3_torrent_url(self):
        return "http://imgjam.com/torrents/album/%d/%d/%d-ogg3.torrent/" % (self.id%1000, self.id, self.id)

    def ogg3_torrent_url_with_name(self):
        """
        >>> Album.objects.get(id=24).torrent_url_with_name()
        'http://imgjam.com/torrents/album/24/24/24-ogg3.torrent/TriFace%20-%20Premiers%20Jets%20--%20Jamendo%20-%20OGG%20Vorbis%20q7%20-%202004-12-17%20%5Bwww.jamendo.com%5D.torrent'

        Can also get with:
        http://www.jamendo.com/get/album/id/album/p2p/plain/24/?are=ogg3&p2pnet=bittorrent
        http://www.jamendo.com/get/album/id/album/p2p/plain/24/?are=ogg3&p2pnet=ed2k
        http://api.jamendo.com/get2/id+type+class+name+bittorrent+ed2k/file/plain?type=archive&album_id=24&class=ogg3
        http://api.jamendo.com/get2/bittorrent/file/plain?type=archive&album_id=24&class=ogg3
        """
        return self.ogg3_torrent_url() \
            + urllib.quote("%s -- Jamendo - OGG Vorbis q7 - %s [www.jamendo.com].torrent" \
                               % (self.filename.encode('UTF-8'), self.releasedate.strftime("%Y-%m-%d")))

    def ogg3_torrent_url_with_name_html(self):
        return "<a href='%s'>D/L</a>" % (self.ogg3_torrent_url_with_name())
    ogg3_torrent_url_with_name_html.allow_tags = True
    ogg3_torrent_url_with_name_html.short_description = "OV torrent"

    def ogg3_zip_url(self):
        return "http://www.jamendo.com/get/album/id/album/archiverestricted/redirect/%d/?are=ogg3" % (self.id)

    def ogg3_zip_url_html(self):
        return "<a href='http://www.jamendo.com/get/album/id/album/archiverestricted/plain/%d/?are=ogg3'>D/L</a>" \
            % (self.id)
    ogg3_zip_url_html.allow_tags = True
    ogg3_zip_url_html.short_description = "OV zip"

    def __unicode__(self):
        return u"%s by %s" % (self.name, self.artist.name)

class Track(models.Model):
    class Meta:
        ordering = ['name',]

    album = models.ForeignKey("Album")
    name = models.CharField(max_length=255, db_index=True, blank=True, null=True)
    filename = models.CharField(max_length=255, db_index=True, blank=True, null=True,
      help_text="Usual directory name without extension")
    mbgid = models.CharField(max_length=36, db_index=True, blank=True, null=True, \
                                 help_text="musicbrainz ID (may be blank)")
    numalbum = models.IntegerField(help_text="Track number in the album. starts at 1")
    id3genre = models.IntegerField(db_index=True, blank=True, null=True,
      help_text="Standard ID3 genre, 0 to 79 (cf http://www.linuxselfhelp.com/HOWTO/MP3-HOWTO-13.html#ss13.3)")
    license = models.URLField(verify_exists=False, db_index=True, blank=True, null=True)
    tags = models.ManyToManyField("Tag", through="TrackTag")
    # not documented:
    duration = models.FloatField(blank=True, null=True)

    def album_html(self):
        return "<a href='%s'>%s</a>" \
            % (urlresolvers.reverse('admin:djamengo_album_change', args=(self.album.pk,)),
               self.album)
    album_html.allow_tags = True
    album_html.admin_order_field = 'album'

    def __unicode__(self):
        return u"%s: %s (by %s)" % (self.album.name, self.name, self.album.artist.name)

class Tag(models.Model):
    class Meta:
        ordering = ['idstr',]

    idstr = models.CharField(max_length=255, db_index=True, blank=True, null=True)

    def track_count(self):
        return self.track_set.count()
    #track_count.admin_order_field = 'track__count' # aggregation not supported
    track_count.short_description = 'Tagged tracks'

    def __unicode__(self):
        return self.idstr

class TrackTag(models.Model):
    class Meta:
        ordering = ['track', 'tag', ]
        unique_together = (('track', 'tag'),)

    track = models.ForeignKey("Track")
    tag = models.ForeignKey("Tag")
    weight = models.FloatField(help_text="weight always between 0 and 1")

    def __unicode__(self):
        return u"%s is tagged %s (%f)" % (self.track.name, self.tag.idstr, self.weight)
